# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: message.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'message.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from compas_pb.generated import message_pb2 as compas__pb_dot_generated_dot_message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rmessage.proto\x12\ncompas_eve\x1a!compas_pb/generated/message.proto\"|\n\x07Message\x12+\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x1d.compas_eve.Message.DataEntry\x1a\x44\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.compas_pb.data.AnyData:\x02\x38\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'message_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MESSAGE_DATAENTRY']._loaded_options = None
  _globals['_MESSAGE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGE']._serialized_start=64
  _globals['_MESSAGE']._serialized_end=188
  _globals['_MESSAGE_DATAENTRY']._serialized_start=120
  _globals['_MESSAGE_DATAENTRY']._serialized_end=188
# @@protoc_insertion_point(module_scope)
